@@ -131,21 +131,6 @@ let serve_local name ims env =
       if should_deny name then deny name
       else Missing
 
-let make_directory path =
-  let rec loop cwd = function
-    | dir :: rest ->
-        let name = cwd ^/ dir in
-        if not (Sys.file_exists name) then
-          mkdir name 0o755
-        else if not (Sys.is_directory name) then
-          failwith ("file " ^ name ^ " is not a directory");
-        loop name rest
-    | [] -> ()
-  in
-  match explode_path path with
-  | "" :: dirs -> loop "/" dirs
-  | dirs -> loop "." dirs
-
 let create_hint name =
   make_directory (Filename.dirname name);
   close (openfile (in_progress name) [O_CREAT; O_WRONLY] 0o644)
@@ -202,7 +187,7 @@ let close_cache cache size mod_time =
       end else begin
         error_message "Size of %s should be %Ld, not %Ld"
           file size (file_size tmp_file);
-        Sys.remove tmp_file;
+        rm tmp_file;
         raise Wrong_size
       end
   | Pass_through -> ()
@@ -213,7 +198,7 @@ let remove_cache cache =
   | Cache { tmp_file = tmp_file; chan = chan } ->
       close_out chan;
       error_message "Removing %s (size: %Ld)" tmp_file (file_size tmp_file);
-      Sys.remove tmp_file
+      rm tmp_file
   | Pass_through | Undefined -> ()
 
 type download_status =
@@ -485,28 +470,27 @@ let serve_file env =
       | Missing -> cache_miss url name ims 0.
   with Failure msg | Invalid_argument msg-> forbidden msg
 
-let proxy_service =
-  object
-    method name = "proxy_service"
-    method def_term = `Proxy_service
-    method print fmt = Format.fprintf fmt "%s" "proxy_service"
-    method process_header env =
-      debug_message "Connection from %s"
-        (Server.remote_address env#remote_socket_addr ~with_port: true);
-      if env#cgi_request_method = "GET" && env#cgi_query_string = "" then
-        serve_file env
-      else begin
-        debug_headers (sprintf "Request %s" env#cgi_request_uri)
-          env#input_header_fields;
-        forbidden "invalid HTTP request"
-      end
+let process_header env =
+  debug_message "Connection from %s"
+    (string_of_sockaddr env#remote_socket_addr ~with_port: true);
+  if env#cgi_request_method = "GET" && env#cgi_query_string = "" then
+    serve_file env
+  else begin
+    debug_headers (sprintf "Request %s" env#cgi_request_uri)
+      env#input_header_fields;
+    forbidden "invalid HTTP request"
   end
 
-let server s =
-  Sys.chdir cache_dir;
+let server sockets =
   info_message "Version: %s" Version.number;
   print_config (info_message "%s");
-  Server.loop s proxy_service
+  Server.loop sockets
+    (object
+       method name = "proxy_service"
+       method def_term = `Proxy_service
+       method print fmt = Format.fprintf fmt "%s" "proxy_service"
+       method process_header = process_header
+     end)
 
 let daemonize proc x =
   ignore (setsid ());
@@ -516,11 +500,13 @@ let daemonize proc x =
   if fork () = 0 && fork () = 0 then
     proc x
 
-let () =
-  try
-    let s = Server.init ~user ~group ~interface ~port in
-    if !foreground then server s
-    else daemonize server s
-  with e ->
-    error_message "%s" (string_of_exception e);
-    exit 1
+let approx () =
+  match Server.bind ~interface ~port with
+  | [] -> failwith "no sockets created"
+  | sockets ->
+      drop_privileges ~user ~group;
+      Sys.chdir cache_dir;
+      if !foreground then server sockets
+      else daemonize server sockets
+
+let () = main_program approx ()
