@@ -1,10 +1,10 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2007  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2008  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
 open Printf
 open Util
-open Default_config
+open Config
 open Log
 
 let string_of_time t =
@@ -24,15 +24,13 @@ let translate_request url =
   let path = relative_url url in
   match explode_path path with
   | dist :: rest ->
-      (try implode_path (Config.get dist :: rest), path
+      (try implode_path (Config_file.get dist :: rest), path
        with Not_found -> failwith ("no remote repository for " ^ dist))
   | [] ->
       invalid_arg "translate_request"
 
 let translate_file file =
-  let dist, path = split_cache_path file in
-  try Config.get dist ^/ path
-  with Not_found -> invalid_arg ("translate_file " ^ file)
+  let dist, path = split_cache_path file in Config_file.get dist ^/ path
 
 type protocol = HTTP | FTP | FILE
 
@@ -52,7 +50,7 @@ let rate_option =
   | str -> "--limit-rate " ^ str
 
 let curl_command options url =
-  sprintf "/usr/bin/curl --fail --silent --location %s %s %s"
+  sprintf "/usr/bin/curl --fail --silent --header \"Pragma: no-cache\" %s %s %s"
     rate_option (String.concat " " options) (quoted_string url)
 
 let head_command = curl_command ["--head"]
@@ -65,25 +63,20 @@ let iter_headers proc chan =
   let rec loop () =
     match next () with
     | Some header ->
-	let n = String.length header in
-	if n > 0 && header.[n - 1] = '\r' then
-	  if n > 1 then
-	    begin
-	      proc (String.sub header 0 (n - 1));
-	      loop ()
-	    end
-	  else
-	    (* CRLF terminates headers *)
-	    ()
-	else
-	  error_message "Unexpected header: %s" header
+        let n = String.length header in
+        if n > 0 && header.[n - 1] = '\r' then
+          if n > 1 then begin
+            proc (String.sub header 0 (n - 1));
+            loop ()
+          end else () (* CRLF terminates headers *)
+        else error_message "Unexpected header: %s" header
     | None -> ()
   in
   loop ()
 
 let head url callback =
   let cmd = head_command url in
-  if debug then debug_message "Command: %s" cmd;
+  debug_message "Command: %s" cmd;
   with_process cmd ~error: url (iter_headers callback)
 
 let download_command headers header_callback =
@@ -109,7 +102,7 @@ let seq f g x = (f x; g x)
 
 let download url ?(headers=[]) ?header_callback callback =
   let cmd = download_command headers header_callback url in
-  if debug then debug_message "Command: %s" cmd;
+  debug_message "Command: %s" cmd;
   with_process cmd ~error: url
     (match header_callback with
      | Some proc -> seq (iter_headers proc) (iter_body callback)
@@ -118,18 +111,18 @@ let download url ?(headers=[]) ?header_callback callback =
 let download_file file =
   let file' = gensym file in
   let options =
-    [ "--output"; file'; "--remote-time" ] @
-      if Sys.file_exists file then
-	[ "--time-cond"; quoted_string (string_of_time (file_modtime file)) ]
-      else []
+    ["--output"; file'; "--remote-time";
+     "--location"; "--max-redirs"; string_of_int max_redirects] @
+    (if Sys.file_exists file then
+       ["--time-cond"; quoted_string (string_of_time (file_modtime file))]
+     else [])
   in
   let cmd = curl_command options (translate_file file) in
-  if debug then debug_message "Command: %s" cmd;
+  debug_message "Command: %s" cmd;
   if Sys.command cmd = 0 then
     (* file' may not exist if file was not modified *)
     try Sys.rename file' file with _ -> ()
-  else
-    begin
-      rm file';
-      failwith ("cannot download " ^ file)
-    end
+  else begin
+    rm file';
+    failwith ("cannot download " ^ file)
+  end
