@@ -1,5 +1,5 @@
 (* approx: proxy server for Debian archive files
-   Copyright (C) 2007  Eric C. Cooper <ecc@cmu.edu>
+   Copyright (C) 2008  Eric C. Cooper <ecc@cmu.edu>
    Released under the GNU General Public License *)
 
 (* Check if the first string is a prefix of the second *)
@@ -32,14 +32,19 @@ val explode_path : string -> string list
 
 val implode_path : string list -> string
 
-(* Return a quoted string *)
-
-val quoted_string : string -> string
-
 (* Infix operator to concatenate two pathname components *)
 
 val (^/) : string -> string -> string
 
+(* Create a directory, including any intermediate directories
+   along the specified path (like "mkdir --parents") *)
+
+val make_directory : string -> unit
+
+(* Return a quoted string *)
+
+val quoted_string : string -> string
+
 (* Return the relative portion of a pathname *)
 
 val relative_path : string -> string
@@ -61,6 +66,10 @@ val without_extension : string -> string
 
 val extension : string -> string
 
+(* Return the underlying value of an option, otherwise raise Not_found *)
+
+val the : 'a option -> 'a
+
 (* Call a function making sure that a cleanup procedure is called
    before returning the result of the function or raising an exception *)
 
@@ -183,10 +192,26 @@ val file_sha256sum : string -> string
 
 val drop_privileges : user:string -> group:string -> unit
 
+(* Check that the program is executing as the given user and group *)
+
+val check_id : user:string -> group:string -> unit
+
+(* Convert a socket address to a string *)
+
+val string_of_sockaddr : Unix.sockaddr -> with_port:bool -> string
+
 (* Return a descriptive message for an exception *)
 
 val string_of_exception : exn -> string
 
-(* Run the main function of a program and print any uncaught exceptions *)
+(* Call a procedure and print, but otherwise ignore, any exception *)
+
+val perform : ('a -> unit) -> 'a -> unit
+
+(* Run the main function of a program and print any uncaught exception *)
 
 val main_program : ('a -> unit) -> 'a -> unit
+
+(* Conditionally print on stderr *)
+
+val print_if : bool -> ('a, unit, string, unit) format4 -> 'a
